==> ovm1.01.tgz: OpenVM/src/syslib/user/ovm_classpath/java/lang/VMClassLoader.java <==
/* VMClassLoader.java -- Reference implementation of native interface
   required by ClassLoader
   Copyright (C) 1998, 2001, 2002 Free Software Foundation

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package java.lang;

import java.security.ProtectionDomain;
import java.net.URL;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import java.lang.reflect.Constructor;
import gnu.java.lang.SystemClassLoader;

import gnu.java.util.EmptyEnumeration;

/**
 * java.lang.VMClassLoader is a package-private helper for VMs to implement
 * on behalf of java.lang.ClassLoader.
 *
 * <h3>OVM To-DO</h3>
 * <p>Methods related to dynamic class loading throw an 
 * <tt>UnsupportedOperationException</tt>.
 * <p>The <tt>Class</tt> object retrieval methods have been implemented.
 * <p>Resources are always null.
 * <p>It is unclear how/when this class will be used in the initialization 
 * process.
 * <p>There is no Package support.
 *
 * @author David Holmes (OVM)
 * @author John Keiser
 * @author Mark Wielaard <mark@klomp.org>
 * @author Eric Blake <ebb9@email.byu.edu>
 */
final class VMClassLoader
{
  /**
   * Helper to define a class using a string of bytes. This assumes that
   * the security checks have already been performed, if necessary.
   * <strong>This method will be removed in a future version of GNU
   * Classpath</strong>.
   *
   * @param name the name to give the class, or null if unknown
   * @param data the data representing the classfile, in classfile format
   * @param offset the offset into the data where the classfile starts
   * @param len the length of the classfile data in the array
   * @return the class that was defined
   * @throws ClassFormatError if data is not in proper classfile format
   * @deprecated Implement
   * {@link #defineClass(ClassLoader, String, byte[], int, int, ProtectionDomain)}
   *   instead.
   * FIXME: Why do ProtectionDomains need to be implemented in native
   * code, anyway?
   */
  static final Class defineClass(ClassLoader cl, String name,
                                        byte[] data, int offset, int len)
      throws ClassFormatError
  {
      if (name != null && cl != bootClassLoader && name.startsWith("java."))
	  throw new SecurityException();
      if (offset < 0 || offset + len > data.length)
	  throw new IndexOutOfBoundsException();
      return LibraryImports.defineClass(cl, name, data, offset, len);
  }

  static final void register(ClassLoader l) {
    if (!(l instanceof ImageClassLoader))
      LibraryImports.setPeer(null, l);
  }

  static final Class findLoadedClass(ClassLoader cl, String name) {
    return LibraryImports.findLoadedClass(cl, name);
  }

  /**
   * Helper to define a class using a string of bytes. This assumes that
   * the security checks have already been performed, if necessary.
   *
   * <strong>For backward compatibility, this just ignores the protection
   * domain; that is the wrong behavior, and you should directly implement
   * this method natively if you can.</strong>
   *
   * @param name the name to give the class, or null if unknown
   * @param data the data representing the classfile, in classfile format
   * @param offset the offset into the data where the classfile starts
   * @param len the length of the classfile data in the array
   * @param pd the protection domain
   * @return the class that was defined
   * @throws ClassFormatError if data is not in proper classfile format
   */
  static final Class defineClass(ClassLoader cl, String name,
                                 byte[] data, int offset, int len,
                                 ProtectionDomain pd)
    throws ClassFormatError
  {
      // XXX: Can we handle protection domains at this level?
      return defineClass(cl, name, data, offset, len);
  }

  /**
   * Helper to resolve all references to other classes from this class.
   *
   * @param c the class to resolve
   */
    static final void resolveClass(Class c) {
        // FIXME: it's possible to bring a class into the image which will
        // not resolve due to a missing referred type. We should have some
        // means of checking this and throwing the right exception at the
        // right time. For now we assume success
    }

  /**
   * Helper to load a class from the bootstrap class loader.  Not used.
   *
   * @param name the class name to load
   * @param resolve whether to resolve it
   * @return the class, loaded by the system classloader
   */
  static final Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException
  {
      Class c = bootClassLoader.findLoadedClass(name);
      if (c != null)
	  return c;
      c = bootClassLoader.findClass(name);
      if (resolve)
	  bootClassLoader.resolveClass(c);
      return c;
  }

  /**
   * Helper to load a resource from the bootstrap class loader.  Not
   * used.
   *
   * @param name the resource to find
   * @return the URL to the resource
   */
  static URL getResource(String name)
  {
      return bootClassLoader.findResource(name);
  }

  /**
   * Helper to get a list of resources from the bootstrap class loader.
   *
   * @param name the resource to find
   * @return an enumeration of resources
   * @throws IOException if one occurs
   */
  static Enumeration getResources(String name) throws IOException
  {
      return bootClassLoader.findResources(name);
  }

    // ClassLoader.definedPackages is not accessible from the type
    // ImageClassLoader, calling this method performs the required downcast
    static Map getDefinedPackages(ClassLoader cl) {
	return cl.definedPackages;
    }
  /**
   * Helper to get a package from the bootstrap class loader.  The default
   * implementation of returning null may be adequate, or you may decide
   * that this needs some native help.
   *
   * @param name the name to find
   * @return the named package, if it exists
   */
  static Package getPackage(String name)
  {
      return (Package) getDefinedPackages(bootClassLoader).get(name);
  }

  /**
   * Helper to get all packages from the bootstrap class loader.  The default
   * implementation of returning an empty array may be adequate, or you may
   * decide that this needs some native help.
   *
   * @return all named packages, if any exist
   */
  static Package[] getPackages()
  {
      Package[] ret = new Package[getDefinedPackages(bootClassLoader).size()];
      getDefinedPackages(bootClassLoader).values().toArray(ret);
      return ret;
  }

    static final String typeString = "ZBCDFIJSV";
  /**
   * Helper for java.lang.Integer, Byte, etc to get the TYPE class
   * at initialization time. The type code is one of the chars that
   * represents the primitive type as in JNI.
   *
   * <ul>
   * <li>'Z' - boolean</li>
   * <li>'B' - byte</li>
   * <li>'C' - char</li>
   * <li>'D' - double</li>
   * <li>'F' - float</li>
   * <li>'I' - int</li>
   * <li>'J' - long</li>
   * <li>'S' - short</li>
   * <li>'V' - void</li>
   * </ul>
   *
   *
   * @param type the primitive type
   * @return a "bogus" class representing the primitive type
   */
  static final Class getPrimitiveClass(char type) {
      if (typeString.indexOf(type) == -1) {
        throw new NoClassDefFoundError("Invalid type specifier: " + type);
      }
      return LibraryImports.getPrimitiveClass(type);
  }

  /**
   * The system default for assertion status. This is used for all system
   * classes (those with a null ClassLoader), as well as the initial value for
   * every ClassLoader's default assertion status.
   *
   *
   * @return the system-wide default assertion status
   */
  static final boolean defaultAssertionStatus()
  {
    return true;
  }

  /**
   * The system default for package assertion status. This is used for all
   * ClassLoader's packageAssertionStatus defaults. It must be a map of
   * package names to Boolean.TRUE or Boolean.FALSE, with the unnamed package
   * represented as a null key.
   *
   * XXX - Not implemented yet; this requires native help.
   *
   * @return a (read-only) map for the default packageAssertionStatus
   */
  static final Map packageAssertionStatus()
  {
    return new HashMap();
  }

  /**
   * The system default for class assertion status. This is used for all
   * ClassLoader's classAssertionStatus defaults. It must be a map of
   * class names to Boolean.TRUE or Boolean.FALSE
   *
   * XXX - Not implemented yet; this requires native help.
   *
   * @return a (read-only) map for the default classAssertionStatus
   */
  static final Map classAssertionStatus()
  {
    return new HashMap();
  }

    static ImageClassLoader bootClassLoader;

  // OVM specific version
  static ClassLoader getSystemClassLoader() {
    bootClassLoader =
      new ImageClassLoader(null,
			   LibraryImports.getBootContext(),
			   LibraryImports.getBootClassPath());
    ImageClassLoader dsysCl =
      new ImageClassLoader(bootClassLoader,
			   LibraryImports.getSystemContext(),
			   System.getProperty("java.class.path"));
    bootClassLoader.addURLs(LibraryImports.getBootClassPath());
    dsysCl.addURLs(System.getProperty("java.class.path"));

    // This method is called as the initialization of systemClassLoader,
    // so if there is a null value, this is the first call and we must check
    // for java.system.class.loader.
    String loader = System.getProperty("java.system.class.loader");
    if (loader != null) {
      try {
	Class cl = bootClassLoader.loadClass("java.lang.ClassLoader");
	Class sysCl = dsysCl.loadClass(loader);
	if (!cl.isAssignableFrom(sysCl))
	  System.err.println(loader + " is not a ClassLoader"
			     + " using default system class loader");
	else {
	  Constructor c = sysCl.getConstructor(new Class[] { cl } );
	  return (ClassLoader) c.newInstance(new Object[] { dsysCl });
	}
      }
      catch (Exception e) {
	System.err.println("Requested system classloader "
			   + loader + " failed, "
			   + "using default system classloader");
	e.printStackTrace();
      }
    }
       
    // Fallback to OVM SystemClassLoader.
    return dsysCl;
  }
}

==> ovm1.4.tgz: OpenVM/src/syslib/user/ovm_classpath/java/lang/VMClassLoader.java <==
/* VMClassLoader.java -- Reference implementation of native interface
   required by ClassLoader
   Copyright (C) 1998, 2001, 2002 Free Software Foundation

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package java.lang;

import java.security.ProtectionDomain;
import java.net.URL;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import java.lang.reflect.Constructor;

import gnu.java.util.EmptyEnumeration;

/**
 * java.lang.VMClassLoader is a package-private helper for VMs to implement
 * on behalf of java.lang.ClassLoader.<p>
 *
 * In Ovm, classes are pre-loaded into the bootstrap and system
 * classloaders at image-build time.  VMClassLoader defines bootstrap
 * and system classloader objects that know to check whether a class
 * is pre-loaded before going off and searching for it.<p>
 *
 * In most JVMs, the bootstrap classloader is not a {@link ClassLoader}
 * object at all, but some special code written in the VM's
 * implementation language.  Ovm can do things differently, because we
 * rely on static compilation.<p>
 *
 * VMClassLoader is critical to the whole bootstrapping process.
 * <ul>
 * <li> No class that uses assertions can be initialized before the
 *      bootstrap classloader has been created.  This implies that
 *      there can be no assertions here or in 
 * <li> Our static intializer uses the <code>path.seperator</code>
 *      property, so it will trigger initialization of system
 *      properties.
 * <li> The main thread calls {@link #getSystemClassLoader} to
 *      initialize it's classloading context.
 * <li> Depending on the default locale, we may end up calling
 *      {@link getResouce} to convert command-line arguments to
 *      strings while computing the default system propteries.
 * <li> By the time {@link #getSystemClassLoader} starts, we had
 *      better have a complete set of default system properties,
 *      because we need <code>java.class.path</code> and
 *      <code>java.system.class.loader</code>.
 * </ul>
 *
 * <h3>OVM To-DO</h3>
 * <ul>
 * <li> Assertion status.
 * <li> Other things that where not implemented initially, and have
 *      not yet been tripped over ({@link #resolveClass}, for one).
 * </ul>
 *
 * @see ImageClassLoader
 *
 * @author David Holmes (OVM)
 * @author Jason Baker (OVM)
 * @author John Keiser
 * @author Mark Wielaard <mark@klomp.org>
 * @author Eric Blake <ebb9@email.byu.edu>
 */
final class VMClassLoader
{
  /**
   * Helper to define a class using a string of bytes. This assumes that
   * the security checks have already been performed, if necessary.
   * <strong>This method will be removed in a future version of GNU
   * Classpath</strong>.
   *
   * @param name the name to give the class, or null if unknown
   * @param data the data representing the classfile, in classfile format
   * @param offset the offset into the data where the classfile starts
   * @param len the length of the classfile data in the array
   * @return the class that was defined
   * @throws ClassFormatError if data is not in proper classfile format
   * @deprecated Implement
   * {@link #defineClass(ClassLoader, String, byte[], int, int, ProtectionDomain)}
   *   instead.
   * FIXME: Why do ProtectionDomains need to be implemented in native
   * code, anyway?
   */
  static final Class defineClass(ClassLoader cl, String name,
                                        byte[] data, int offset, int len)
      throws ClassFormatError
  {
      if (name != null && cl != bootClassLoader && name.startsWith("java."))
	  throw new SecurityException();
      if (offset < 0 || offset + len > data.length)
	  throw new IndexOutOfBoundsException();
      return LibraryImports.defineClass(cl, name, data, offset, len);
  }

  static final void register(ClassLoader l) {
    if (!(l instanceof ImageClassLoader))
      LibraryImports.setPeer(null, l);
  }

  static final Class findLoadedClass(ClassLoader cl, String name) {
    return LibraryImports.findLoadedClass(cl, name);
  }

  /**
   * Helper to define a class using a string of bytes. This assumes that
   * the security checks have already been performed, if necessary.
   *
   * <strong>For backward compatibility, this just ignores the protection
   * domain; that is the wrong behavior, and you should directly implement
   * this method natively if you can.</strong>
   *
   * @param name the name to give the class, or null if unknown
   * @param data the data representing the classfile, in classfile format
   * @param offset the offset into the data where the classfile starts
   * @param len the length of the classfile data in the array
   * @param pd the protection domain
   * @return the class that was defined
   * @throws ClassFormatError if data is not in proper classfile format
   */
  static final Class defineClass(ClassLoader cl, String name,
                                 byte[] data, int offset, int len,
                                 ProtectionDomain pd)
    throws ClassFormatError
  {
      // XXX: Can we handle protection domains at this level?
      return defineClass(cl, name, data, offset, len);
  }

  /**
   * Helper to resolve all references to other classes from this class.
   *
   * @param c the class to resolve
   */
    static final void resolveClass(Class c) {
        // FIXME: it's possible to bring a class into the image which will
        // not resolve due to a missing referred type. We should have some
        // means of checking this and throwing the right exception at the
        // right time. For now we assume success
    }

  /**
   * Helper to load a class from the bootstrap class loader.  Not used.
   *
   * @param name the class name to load
   * @param resolve whether to resolve it
   * @return the class, loaded by the system classloader
   */
  static final Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException
  {
      Class c = bootClassLoader.findLoadedClass(name);
      if (c != null)
	  return c;
      c = bootClassLoader.findClass(name);
      if (resolve)
	  bootClassLoader.resolveClass(c);
      return c;
  }

  /**
   * Helper to load a resource from the bootstrap class loader.  Not
   * used.
   *
   * @param name the resource to find
   * @return the URL to the resource
   */
  static URL getResource(String name)
  {
      return bootClassLoader.findResource(name);
  }

  /**
   * Helper to get a list of resources from the bootstrap class loader.
   *
   * @param name the resource to find
   * @return an enumeration of resources
   * @throws IOException if one occurs
   */
  static Enumeration getResources(String name) throws IOException
  {
      return bootClassLoader.findResources(name);
  }

    // ClassLoader.definedPackages is not accessible from the type
    // ImageClassLoader, calling this method performs the required downcast
    static Map getDefinedPackages(ClassLoader cl) {
	return cl.definedPackages;
    }
  /**
   * Helper to get a package from the bootstrap class loader.  The default
   * implementation of returning null may be adequate, or you may decide
   * that this needs some native help.
   *
   * @param name the name to find
   * @return the named package, if it exists
   */
  static Package getPackage(String name)
  {
      return (Package) getDefinedPackages(bootClassLoader).get(name);
  }

  /**
   * Helper to get all packages from the bootstrap class loader.  The default
   * implementation of returning an empty array may be adequate, or you may
   * decide that this needs some native help.
   *
   * @return all named packages, if any exist
   */
  static Package[] getPackages()
  {
      Package[] ret = new Package[getDefinedPackages(bootClassLoader).size()];
      getDefinedPackages(bootClassLoader).values().toArray(ret);
      return ret;
  }

    static final String typeString = "ZBCDFIJSV";
  /**
   * Helper for java.lang.Integer, Byte, etc to get the TYPE class
   * at initialization time. The type code is one of the chars that
   * represents the primitive type as in JNI.
   *
   * <ul>
   * <li>'Z' - boolean</li>
   * <li>'B' - byte</li>
   * <li>'C' - char</li>
   * <li>'D' - double</li>
   * <li>'F' - float</li>
   * <li>'I' - int</li>
   * <li>'J' - long</li>
   * <li>'S' - short</li>
   * <li>'V' - void</li>
   * </ul>
   *
   *
   * @param type the primitive type
   * @return a "bogus" class representing the primitive type
   */
  static final Class getPrimitiveClass(char type) {
      if (typeString.indexOf(type) == -1) {
        throw new NoClassDefFoundError("Invalid type specifier: " + type);
      }
      return LibraryImports.getPrimitiveClass(type);
  }

  /**
   * The system default for assertion status. This is used for all system
   * classes (those with a null ClassLoader), as well as the initial value for
   * every ClassLoader's default assertion status.
   *
   *
   * @return the system-wide default assertion status
   */
  static final boolean defaultAssertionStatus()
  {
    return true;
  }

  /**
   * The system default for package assertion status. This is used for all
   * ClassLoader's packageAssertionStatus defaults. It must be a map of
   * package names to Boolean.TRUE or Boolean.FALSE, with the unnamed package
   * represented as a null key.
   *
   * XXX - Not implemented yet; this requires native help.
   *
   * @return a (read-only) map for the default packageAssertionStatus
   */
  static final Map packageAssertionStatus()
  {
    return new HashMap();
  }

  /**
   * The system default for class assertion status. This is used for all
   * ClassLoader's classAssertionStatus defaults. It must be a map of
   * class names to Boolean.TRUE or Boolean.FALSE
   *
   * XXX - Not implemented yet; this requires native help.
   *
   * @return a (read-only) map for the default classAssertionStatus
   */
  static final Map classAssertionStatus()
  {
    return new HashMap();
  }

    // Initialize the boot classloader early.  We do NOT access the
    // org.ovmj.boot.class.path system property to avoid any
    // initialization cycles.  Depending on the locale, getResource
    // may be called to translate commandline arguments into UTF16,
    // and we need that to work for -classpath or -Djava.class.path to
    // have some effect
    static ImageClassLoader bootClassLoader;
    static {
	String bootClassPath = LibraryImports.getBootClassPath();
	bootClassLoader =
      	    new ImageClassLoader(null,
				 LibraryImports.getBootContext(),
				 bootClassPath);
	// I beleive that we must call addURLs only after
	// bootClassLoader has been initialized.  We are still well
	// away from the point where dynamic loading can actually
	// start, but getResource is another matter.
	bootClassLoader.addURLs(bootClassPath);
    }
	
    // OVM specific version -- the default system classloader is a child
    // of the real live bootClassLoader object.  By the time the first
    // line of this method returns, we better have fully initialized the
    // system properties
    static ClassLoader getSystemClassLoader() {
	String cp = System.getProperty("java.class.path");
	// Assert will not work until VMClassLoader.<clinit>() has
	// run to completion.
	if (cp == null)
	    throw new Error("java.class.path not set");
	ImageClassLoader dsysCl =
      	    new ImageClassLoader(bootClassLoader,
				 LibraryImports.getSystemContext(),
				 cp);
	dsysCl.addURLs(cp);

	// This method is called as the initialization of systemClassLoader,
	// so if there is a null value, this is the first call and we must check
	// for java.system.class.loader.
	String loader = System.getProperty("java.system.class.loader");
	if (loader != null) {
	    try {
		Class cl = bootClassLoader.loadClass("java.lang.ClassLoader");
		Class sysCl = dsysCl.loadClass(loader);
		if (!cl.isAssignableFrom(sysCl))
		    System.err.println(loader + " is not a ClassLoader"
				       + " using default system class loader");
		else {
		    Constructor c = sysCl.getConstructor(new Class[] { cl } );
		    return (ClassLoader) c.newInstance(new Object[] { dsysCl });
		}
	    }
	    catch (Exception e) {
		System.err.println("Requested system classloader "
				   + loader + " failed, "
				   + "using default system classloader");
		e.printStackTrace();
	    }
	}
       
      // Fallback to OVM SystemClassLoader.
      return dsysCl;
  }
}
